@@ -1,44 +1,82 @@
 #!/bin/bash
-# $Id: install.sh,v 1.8 2005/06/27 10:41:17 xmldoc Exp $
-# $Source: /cvsroot/docbook/releasetools/install.sh,v $ #
+# $Id: install.sh 6974 2007-07-08 20:36:58Z xmldoc $
+# $Source$ #
 
-# install.sh - Set up catalogs & locating rules for a XML/XSLT distribution
+# install.sh - Set up user environment for a XML/XSLT distribution
 
-# This is as a interactive installer for updating a single-user
-# environment to make use of XML catalog and schema "locating
-# rules" data provided in an XML/XSLT distribution.
+# This is as an interactive installer for updating your
+# environment to use an XML/XSLT distribution such as the DocBook
+# XSL Stylesheets. Its main purpose is to configure your
+# environment with XML catalog data and schema "locating rules"
+# data provided in the XML/XSLT distribution.
 #
 # Although this installer was created for the DocBook project, it
 # is a general-purpose tool that can be used with any XML/XSLT
 # distribution that provides XML/SGML catalogs and locating rules.
 #
-# It is mainly intended to make things easier for users who want
-# to install a particular XML/XSLT distribution that has not (yet)
-# been packaged for their OS distro (Debian, Fedora, whatever).
+# This script is mainly intended to make things easier for you if
+# you want to install a particular XML/XSLT distribution that has
+# not (yet) been packaged for your OS distro (Debian, Fedora,
+# whatever), or to use "snapshot" or development releases 
 #
-# It works by updating the user's shell startup files (e.g.,
-# .bashrc and .cshrc) and .emacs file and by finding or creating a
-# writable CatalogManager.properties file to update.
+# It works by updating your shell startup file (e.g., .bashrc and
+# .cshrc) and .emacs file and by finding or creating a writable
+# CatalogManager.properties file to update.
 #
 # It makes backup copies of any files it touches, and also
 # generates a uninstall.sh script for reverting its changes.
 #
-# In the same directory from it is run, it expects to find four
-# files (below). And if it is unable to locate a
-# CatalogManager.properties file in the user environment, it
-# expects to find an "example" one in the same directory, which it
-# copies over to the user's ~/.resolver directory.
+# In the same directory where it is located, it expects to find
+# the following four files:
+#   - locatingrules.xml
+#   - catalog.xml
+#   - catalog
+#   - .urilist
+# And if it's unable to locate a CatalogManager.properties file in
+# your environment, it expects to find an "example" one in the
+# same directory as itself, which it copies over to your
+# ~/.resolver directory.
+#
+# If the distribution contains any executables, change the value
+# of the thisBinDir to a colon-separated list of the pathnames of
+# the directories that contain those executables.
 
-thisLocatingRules=$PWD/locatingrules.xml
-thisXmlCatalog=$PWD/catalog.xml
-thisSgmlCatalog=$PWD/catalog
+mydir=$(readlink -f $(dirname $0))
+thisLocatingRules=$mydir/locatingrules.xml
+thisXmlCatalog=$mydir/catalog.xml
+thisSgmlCatalog=$mydir/catalog
 # .urilist file contains a list of pairs of local pathnames and
 # URIs to test for catalog resolution
-thisUriList=$PWD/.urilist
-
-exampleCatalogManager=$PWD/.CatalogManager.properties.example 
+thisUriList=$mydir/.urilist
+exampleCatalogManager=$mydir/.CatalogManager.properties.example
 thisCatalogManager=$HOME/.resolver/CatalogManager.properties
 
+# thisBinDir directory is a colon-separated list of the pathnames
+# to all directories that contain executables provided with the
+# distribution (for example, the DocBook XSL Stylesheets
+# distribution contains a "docbook-xsl-update" convenience script
+# for rsync'ing up to the latest docbook-xsl snapshot). The
+# install.sh script adds the value of thisBinDir to your PATH
+# environment variable
+thisBinDir=$mydir/tools/bin
+
+emit_message() {
+  echo "$1" 1>&2
+}
+
+if [ ! "${*#--batch}" = "$*" ]; then
+  batchmode="Yes";
+else
+  batchmode="No";
+  emit_message
+  if [ ! "$1" = "--test" ]; then 
+    emit_message "NOTE: For non-interactive installs/uninstalls, use --batch"
+    if [ ! "$1" = "--uninstall" ]; then
+      emit_message
+    fi
+  fi
+fi
+
 osName=$(uname -o)
 classPathSeparator=":"
 if [ "$osName" = "Cygwin" ]; then
@@ -62,26 +100,26 @@ main() {
 }
 
 removeOldFiles() {
-  rm -f ./.profile.incl
-  rm -f ./.cshrc.incl
-  rm -f ./.emacs.el
+  rm -f $mydir/.profile.incl
+  rm -f $mydir/.cshrc.incl
+  rm -f $mydir/.emacs.el
 }
 
 checkRoot() {
   if [ $(id -u)  == "0" ]; then
-    cat <<EOF
+    cat 1>&2 <<EOF
 
 WARNING: This install script is meant to be run as a non-root
          user, but you are running it as root.
 
 EOF
     read -s -n1 -p "Are you sure you want to continue? [No] "
-    echo "$REPLY"
+    emit_message "$REPLY"
     case $REPLY in
       [yY])
-      echo
+      emit_message
       ;;
-      *) echo "OK, exiting without making changes."
+      *) emit_message "OK, exiting without making changes."
       exit
       ;;
     esac
@@ -97,7 +135,7 @@ updateCatalogManager() {
   #    CatalogManager.properties file found
 
   if [ -z "$CLASSPATH" ]; then
-    cat <<EOF
+    cat 1>&2 <<EOF
 
 NOTE: There is no CLASSPATH variable set in your environment.
       No attempt was made to find a CatalogManager.properties
@@ -135,29 +173,37 @@ EOF
     if [ -f "$existingCatalogManager" ]; then
       # a non-writable CatalogManager.properties exists, so emit a
       # note saying that it won't be used
-      cat <<EOF
+      cat 1>&2 <<EOF
 NOTE: $existingCatalogManager file found,
       but you don't have permission to write to it.
-      Will instead use $thisCatalogManager
+      Will instead use:
+      $thisCatalogManager
 EOF
     else
       # CLASSPATH is set, but no CatalogManager.properties found
       if [ -n "$CLASSPATH" ]; then
-        cat <<EOF
+        cat 1>&2 <<EOF
 NOTE: No CatalogManager.properties found from CLASSPATH.
-      Will instead use $thisCatalogManager
+      Will instead use:
+      $thisCatalogManager
 EOF
       fi
     fi
+    if [ "$batchmode" = "Yes" ]; then
+      emit_message
+    fi
     # end of check for existing writable CatalogManager.properties
 
     if [ -f $thisCatalogManager ]; then
       myCatalogManager=$thisCatalogManager
     else
-      echo
-      read -s -n1 -p "Create $thisCatalogManager file? [Yes] "
-      echo "$REPLY"
-      echo
+      REPLY=""
+      if [ ! "$batchmode" = "Yes" ]; then
+        emit_message
+        read -s -n1 -p "Create $thisCatalogManager file? [Yes] "
+        emit_message "$REPLY"
+        emit_message
+      fi
       case $REPLY in
         [nNqQ])
         emitNoChangeMsg
@@ -166,8 +212,9 @@ EOF
         if [ ! -d "${thisCatalogManager%/*}" ]; then
           mkdir -p ${thisCatalogManager%/*}
         fi
-        cp ./.CatalogManager.properties.example $thisCatalogManager || exit 1
-        echo "NOTE: $thisCatalogManager file created"
+        cp $mydir/.CatalogManager.properties.example $thisCatalogManager || exit 1
+        emit_message "NOTE: Created the following file:"
+        emit_message "      $thisCatalogManager"
         myCatalogManager=$thisCatalogManager
         ;;
       esac
@@ -182,24 +229,28 @@ EOF
     catalogsLine=$(grep "^catalogs=" $myCatalogManager)
     if [ -f /etc/xml/catalog ] && [ "$osName" != "Cygwin" ] \
       && [ "${catalogsLine#*/etc/xml/catalog*}" = "$catalogsLine" ]; then
-      cat <<EOF
+      cat 1>&2 <<EOF
 
-WARNING: /etc/xml/catalog exists but was not found in the
-         $myCatalogManager file. If the
-         /etc/xml/catalog file has content, you probably should reference
-         it in your $myCatalogManager
-         file. This installer can automatically add it for you,
-         but BE WARNED that once it has been added, the uninstaller
-         for this distribution CANNOT REMOVE IT automatically
-         during uninstall. If you no longer want it included, you
-         will need to remove it manually.
+WARNING: /etc/xml/catalog exists but was not found in:
+         $myCatalogManager
+         If /etc/xml/catalog file has content, you probably
+         should reference it in:
+         $myCatalogManager
+         This installer can automatically add it for you,
+         but BE WARNED that once it has been added, the
+         uninstaller for this distribution CANNOT REMOVE IT
+         automatically during uninstall. If you no longer want
+         it included, you will need to remove it manually.
 
 EOF
-      read -s -n1 -p "Add /etc/xml/catalog to $myCatalogManager? [Yes] "
-      echo "$REPLY"
+      REPLY=""
+      if [ ! "$batchmode" = "Yes" ]; then
+        read -s -n1 -p "Add /etc/xml/catalog to $myCatalogManager? [Yes] "
+        emit_message "$REPLY"
+      fi
       case $REPLY in
         [nNqQ])
-        echo
+        emit_message
         ;;
         *)
         etcXmlCatalog=/etc/xml/catalog
@@ -209,15 +260,19 @@ EOF
 
     catalogBackup="$myCatalogManager.$$.bak"
     if [ ! -w "${myCatalogManager%/*}" ]; then
-      echo
-      echo "WARNING: ${myCatalogManager%/*} directory is not writable."
-      echo
+      emit_message
+      emit_message "WARNING: ${myCatalogManager%/*} directory is not writable."
+      emit_message
       emitNoChangeMsg
     else
-      echo
-      read -s -n1 -p "Add $thisJavaXmlCatalog to $myCatalogManager file? [Yes] "
-      echo "$REPLY"
-      echo
+      REPLY=""
+      if [ ! "$batchmode" = "Yes" ]; then
+        emit_message
+        emit_message "Add $thisJavaXmlCatalog"
+        read -s -n1 -p "to $myCatalogManager file? [Yes] "
+        emit_message "$REPLY"
+        emit_message
+      fi
       case $REPLY in
         [nNqQ])
         emitNoChangeMsg
@@ -225,21 +280,25 @@ EOF
         *)
         if [ "$catalogsLine" ] ; then
           if [ "${catalogsLine#*$thisJavaXmlCatalog*}" != "$catalogsLine" ]; then
-            echo "NOTE: $thisJavaXmlCatalog already in $myCatalogManager"
+            emit_message "NOTE: $thisJavaXmlCatalog"
+            emit_message "      already in:"
+            emit_message "      $myCatalogManager"
           else
             mv $myCatalogManager $catalogBackup || exit 1
             sed "s#^catalogs=\(.*\)\$#catalogs=$thisJavaXmlCatalog;\1;$etcXmlCatalog#" $catalogBackup \
             | sed 's/;\+/;/' | sed 's/;$//' > $myCatalogManager || exit 1
-            echo "NOTE: $myCatalogManager file successfully updated."
-            echo "      Backup written to $catalogBackup"
+            emit_message "NOTE: Successfully updated the following file:"
+            emit_message "      $myCatalogManager"
+            emit_message "      Backup written to:"
+            emit_message "      $catalogBackup"
           fi
         else
           mv $myCatalogManager $catalogBackup || exit 1
           cp $catalogBackup $myCatalogManager
           echo "catalogs=$thisJavaXmlCatalog;$etcXmlCatalog" \
           | sed 's/;\+/;/' | sed 's/;$//' >> $myCatalogManager || exit 1
-          echo "NOTE: \"catalogs=\" line added to $myCatalogManager."
-          echo "      Backup written to $catalogBackup"
+          emit_message "NOTE: \"catalogs=\" line added to $myCatalogManager."
+          emit_message "      Backup written to $catalogBackup"
         fi
         ;;
       esac
@@ -256,8 +315,13 @@ EOF
 
 writeDotFiles() {
   while read; do
-    echo "$REPLY" >> .profile.incl
+    echo "$REPLY" >> $mydir/.profile.incl
   done <<EOF
+# $thisBinDir is not in PATH, so add it
+if [ "\${PATH#*$thisBinDir*}" = "\$PATH" ]; then
+  PATH="$thisBinDir:\$PATH"
+  export PATH
+fi
 if [ -z "\$XML_CATALOG_FILES" ]; then
   XML_CATALOG_FILES="$thisXmlCatalog"
 else
@@ -290,8 +354,12 @@ export SGML_CATALOG_FILES
 EOF
 
 while read; do
-  echo "$REPLY" >> .cshrc.incl
+  echo "$REPLY" >> $mydir/.cshrc.incl
 done <<EOF
+# $thisBinDir is not in PATH, so add it
+if ( "\\\`echo \$PATH | grep -v $thisBinDir\\\`" != "" ) then
+  setenv PATH "$thisBinDir:\$PATH"
+endif
 if ( ! $\?XML_CATALOG_FILES ) then
   setenv XML_CATALOG_FILES "$thisXmlCatalog"
 # $thisXmlCatalog is not in XML_CATALOG_FILES, so add it
@@ -308,19 +376,19 @@ endif
 if ( ! $\?SGML_CATALOG_FILES ) then
   setenv SGML_CATALOG_FILES "$thisSgmlCatalog"
 else if ( "\\\`echo \$SGML_CATALOG_FILES | grep -v $thisSgmlCatalog\\\`" != "" ) then
-  setenv SGML_CATALOG_FILES "$thisSgmlCatalog \$SGML_CATALOG_FILES"
+  setenv SGML_CATALOG_FILES "$thisSgmlCatalog:\$SGML_CATALOG_FILES"
 endif
 endif
 # /etc/SGML/catalog exists but is not in SGML_CATALOG_FILES, so add it
 if ( -f /etc/sgml/catalog && "\\\`echo \$SGML_CATALOG_FILES | grep -v /etc/sgml/catalog\\\`" != "" ) then
-  setenv SGML_CATALOG_FILES "\$SGML_CATALOG_FILES /etc/sgml/catalog"
+  setenv SGML_CATALOG_FILES {\$SGML_CATALOG_FILES}:/etc/sgml/catalog
 endif
 EOF
 
 if [ -n "$myCatalogManager" ]; then
   myCatalogManagerDir=${myCatalogManager%/*}
   while read; do
-    echo "$REPLY" >> .profile.incl
+    echo "$REPLY" >> $mydir/.profile.incl
   done <<EOF
 
 
@@ -336,7 +404,7 @@ export CLASSPATH
 EOF
 
   while read; do
-    echo "$REPLY" >> .cshrc.incl
+    echo "$REPLY" >> $mydir/.cshrc.incl
   done <<EOF
 
 
@@ -352,7 +420,7 @@ EOF
 fi
 
 while read; do
-  echo "$REPLY" >> .emacs.el
+  echo "$REPLY" >> $mydir/.emacs.el
 done <<EOF
 (add-hook
   'nxml-mode-hook
@@ -366,7 +434,8 @@ return 0
 }
 
 updateUserStartupFiles() {
-  cat <<EOF
+  if [ ! "$batchmode" = "Yes" ]; then
+  cat 1>&2 <<EOF
 
 NOTE: To source your environment correctly for using the catalog
       files in this distribution, you need to update one or more
@@ -375,27 +444,33 @@ NOTE: To source your environment correctly for using the catalog
       you can make the changes manually.
 
 EOF
+  else
+    emit_message
+  fi
 
-  # if user is running csh or tcsh, target .cshrc and .tcshrc
-  # files for update; otherwise, target .bash_* and .profiles
+  # if running csh or tcsh, target .cshrc and .tcshrc files for
+  # update; otherwise, target .bash_* and .profiles
 
-  parent=$(ps $PPID | grep "/")
+  parent=$(ps -p $PPID | grep "/")
   if [ "${parent#*csh}" != "$parent" ] || [ "${parent#*tcsh}" != "$parent" ]; then
     myStartupFiles=".cshrc .tcshrc"
-    appendLine="source $PWD/.cshrc.incl"
+    appendLine="source $mydir/.cshrc.incl"
   else
     myStartupFiles=".bash_profile .bash_login .profile .bashrc"
-    appendLine=". $PWD/.profile.incl"
+    appendLine=". $mydir/.profile.incl"
   fi
 
   for file in $myStartupFiles; do
     if [ -f "$HOME/$file" ]; then
       dotFileBackup=$HOME/$file.$$.bak
-      read -s -n1 -p "Update $HOME/$file? [Yes] "
-      echo "$REPLY"
+      REPLY=""
+      if [ ! "$batchmode" = "Yes" ]; then
+        read -s -n1 -p "Update $HOME/$file? [Yes] "
+        emit_message "$REPLY"
+      fi
       case $REPLY in
         [nNqQ])
-        cat <<EOF
+        cat 1>&2 <<EOF
 
 NOTE: No change made to $HOME/$file. You either need
       to add the following line to it, or manually source
@@ -412,16 +487,18 @@ EOF
           mv $HOME/$file $dotFileBackup     || exit 1
           cp $dotFileBackup $HOME/$file     || exit 1
           echo "$appendLine" >> $HOME/$file || exit 1
-          cat <<EOF
+          cat 1>&2 <<EOF
+NOTE: Successfully updated the following file:
+      $HOME/$file 
+      Backup written to:
+      $dotFileBackup
 
-NOTE: $HOME/$file file successfully updated.
-      Backup written to $dotFileBackup
 EOF
         else
-          cat <<EOF
-
-NOTE: $HOME/$file already contains information for this distribution.
-      $HOME/$file not updated.
+          cat 1>&2 <<EOF
+NOTE: The following file already contains information for this
+      distribution, so I did not update it.
+      $HOME/$file
 
 EOF
         fi
@@ -430,8 +507,10 @@ EOF
     fi
   done
   if [ -z "$dotFileBackup" ]; then
-    cat <<EOF
-
+    if [ ! "$batchmode" = "Yes" ]; then
+      emit_message
+    fi
+    cat 1>&2 <<EOF
 NOTE: No shell startup files updated. You can source the
       environment for this distribution manually, each time you
       want to use it, by typing the following.
@@ -444,7 +523,7 @@ EOF
 
 updateUserDotEmacs() {
   if [ -f $thisLocatingRules ]; then
-  cat <<EOF
+  cat 1>&2 <<EOF
 
 NOTE: This distribution includes a "schema locating rules" file
       for Emacs/nXML.  To use it, you should update either your
@@ -454,7 +533,7 @@ NOTE: This distribution includes a "schema locating rules" file
 
 EOF
 
-  emacsAppendLine="(load-file \"$PWD/.emacs.el\")"
+  emacsAppendLine="(load-file \"$mydir/.emacs.el\")"
   myEmacsFile=
   for file in .emacs .emacs.el; do
     if [ -f "$HOME/$file" ]; then
@@ -463,16 +542,19 @@ EOF
     fi
   done
   if [ ! -f "$myEmacsFile" ]; then
-    read -s -n1 -p "No .emacs or .emacs.el file. Create one? [No] "
-    echo "$REPLY"
-    echo
+    REPLY=""
+    if [ ! "$batchmode" = "Yes" ]; then
+      read -s -n1 -p "No .emacs or .emacs.el file. Create one? [No] "
+      emit_message "$REPLY"
+      emit_message
+    fi
     case $REPLY in
       [yY])
       myEmacsFile=$HOME/.emacs
       touch $myEmacsFile
       ;;
       *)
-      cat <<EOF
+      cat 1>&2 <<EOF
 NOTE: No Emacs changes made. To use this distribution with,
       Emacs/nXML, you can create a .emacs file and manually add
       the following line to it, or you can run it as a command
@@ -485,12 +567,15 @@ EOF
     esac
   fi
   if [ -n "$myEmacsFile" ]; then
-    read -s -n1 -p  "Update $myEmacsFile? [Yes] "
-    echo "$REPLY"
-    echo
+    REPLY=""
+    if [ ! "$batchmode" = "Yes" ]; then
+      read -s -n1 -p  "Update $myEmacsFile? [Yes] "
+      emit_message "$REPLY"
+      emit_message
+    fi
     case $REPLY in
       [nNqQ])
-      cat <<EOF
+      cat 1>&2 <<EOF
 
 NOTE: No change made to $myEmacsFile. To use this distribution
       with Emacs/nXML, you can manually add the following line
@@ -508,15 +593,18 @@ EOF
         mv $myEmacsFile $dotEmacsBackup    || exit 1
         cp $dotEmacsBackup $myEmacsFile    || exit 1
         echo "$emacsAppendLine" >> $myEmacsFile || exit 1
-        cat <<EOF
-NOTE: $myEmacsFile file successfully updated.
-      Backup written to $dotEmacsBackup
+        cat 1>&2 <<EOF
+NOTE: Successfully updated the following file:
+      $myEmacsFile
+      Backup written to:
+      $dotEmacsBackup
 EOF
       else
-        cat <<EOF
+        cat 1>&2 <<EOF
 
-NOTE: $myEmacsFile already contains information for this distribution.
-      $myEmacsFile not updated.
+NOTE: The following file already contains information for this
+      distribution, so I did not update it.
+      $myEmacsFile
 
 EOF
       fi
@@ -527,7 +615,8 @@ fi
 }
 
 uninstall() {
-  cat <<EOF
+  if [ ! "$batchmode" = "Yes" ]; then
+  cat 1>&2 <<EOF
 
 NOTE: To "uninstall" this distribution, the changes made to your
       CatalogManagers.properties, startup files, and/or .emacs
@@ -535,13 +624,14 @@ NOTE: To "uninstall" this distribution, the changes made to your
       revert them.  Or, if you prefer, you can revert them manually.
 
 EOF
+  fi
 
   if [ "$osName" = "Cygwin" ]; then
     thisXmlCatalog=$thisJavaXmlCatalog
   fi
 
   # make "escaped" version of PWD to use with sed and grep
-  escapedPwd=$(echo $PWD | sed "s#/#\\\\\/#g")
+  escapedPwd=$(echo $mydir | sed "s#/#\\\\\/#g")
 
   # check to see if a non-empty value for catalogManager was fed
   # to uninstaller.
@@ -551,11 +641,14 @@ EOF
     catalogsLine=$(grep "^catalogs=" $myCatalogManager)
     if [ "$catalogsLine" ] ; then
       if [ "${catalogsLine#*$thisXmlCatalog*}" != "$catalogsLine" ]; then
-        read -s -n1 -p "Revert $myCatalogManager? [Yes] "
-        echo "$REPLY"
+        REPLY=""
+        if [ ! "$batchmode" = "Yes" ]; then
+          read -s -n1 -p "Revert $myCatalogManager? [Yes] "
+          emit_message "$REPLY"
+        fi
         case $REPLY in
           [nNqQ]*)
-          cat <<EOF
+          cat 1>&2 <<EOF
 
 NOTE: No change made to $myCatalogManager. You need to manually
       remove the following path from the "catalog=" line.
@@ -568,21 +661,25 @@ EOF
           mv $myCatalogManager $catalogBackup || exit 1
           sed "s#^catalogs=\(.*\)$thisXmlCatalog\(.*\)\$#catalogs=\1\2#" $catalogBackup \
           | sed 's/;\+/;/' | sed 's/;$//' | sed 's/=;/=/' > $myCatalogManager || exit 1
-          cat <<EOF
+          cat 1>&2 <<EOF
+NOTE: Successfully updated the following file:
+      $myCatalogManager
+      Backup written to:
+      $catalogBackup
 
-NOTE: $myCatalogManager file successfully reverted.
-      Backup written to $catalogBackup
 EOF
           ;;
         esac
       else
-        echo "NOTE: No data for this distribution found in $myCatalogManager"
-        echo
+        emit_message "NOTE: No data for this distribution found in:"
+        emit_message "       $myCatalogManager"
+        emit_message
       fi
     else
-      cat <<EOF
-NOTE: No data for this distribution found in $myCatalogManager
-      So, nothing to revert in $myCatalogManager
+      cat 1>&2 <<EOF
+NOTE: No data for this distribution was found in the following
+      file, so I did not revert it.
+      $myCatalogManager
 EOF
     fi
   fi
@@ -595,17 +692,20 @@ EOF
       revertLine="(load-file \"$escapedPwd\/\.emacs\.el\")"
       loadLine="$(grep "$revertLine" "$myEmacsFile")"
       if [ -n "$loadLine" ]; then
-        echo
-        read -s -n1 -p "Revert $myEmacsFile? [Yes] "
-        echo "$REPLY"
+        emit_message
+        REPLY=""
+        if [ ! "$batchmode" = "Yes" ]; then
+          read -s -n1 -p "Revert $myEmacsFile? [Yes] "
+          emit_message "$REPLY"
+        fi
         case $REPLY in
           [nNqQ]*)
-          cat <<EOF
+          cat 1>&2 <<EOF
 
 NOTE: No change made to $myEmacsFile. You need to manually
 remove the following line.
 
-(load-file \"$PWD/.emacs.el\")
+(load-file \"$mydir/.emacs.el\")
 
 EOF
           ;;
@@ -614,16 +714,18 @@ EOF
           mv $myEmacsFile $dotEmacsBackup       || exit 1
           cp $dotEmacsBackup $myEmacsFile       || exit 1
           sed -i "/$revertLine/d" $myEmacsFile  || exit 1
-          cat <<EOF
-
-NOTE: $myEmacsFile file successfully reverted.
-Backup written to $dotEmacsBackup
+          cat 1>&2 <<EOF
+NOTE: successfully reverted the following file:
+      $myEmacsFile
+      Backup written to:
+      $dotEmacsBackup
 
 EOF
           ;;
         esac
       else
-        echo "NOTE: No data for this distribution found in $myEmacsFile"
+        emit_message "NOTE: No data for this distribution found in:"
+        emit_message "      $myEmacsFile"
       fi
     fi
   fi
@@ -634,21 +736,24 @@ EOF
     if [ -e "$HOME/$file" ]; then
       case $file in
         .tcshrc|.cshrc)
-        revertLine="source $PWD/.cshrc.incl"
+        revertLine="source $mydir/.cshrc.incl"
         revertLineEsc="source $escapedPwd\/\.cshrc\.incl"
         ;;
         *)
-        revertLine=". $PWD/.profile.incl"
+        revertLine=". $mydir/.profile.incl"
         revertLineEsc="\. $escapedPwd\/\.profile\.incl"
         ;;
       esac
       lineExists="$(grep "$revertLineEsc" $HOME/$file )"
       if [ "$lineExists" ]; then
-        read -s -n1 -p "Update $HOME/$file? [Yes] "
-        echo "$REPLY"
+        REPLY=""
+        if [ ! "$batchmode" = "Yes" ]; then
+          read -s -n1 -p "Update $HOME/$file? [Yes] "
+          emit_message "$REPLY"
+        fi
         case $REPLY in
           [nNqQ]*)
-          cat <<EOF
+          cat 1>&2 <<EOF
 
 NOTE: No change made to $HOME/$file. You need to manually remove
       the following line from it.
@@ -662,46 +767,52 @@ EOF
           mv $HOME/$file $dotFileBackup           || exit 1
           cp $dotFileBackup $HOME/$file           || exit 1
           sed -i "/$revertLineEsc/d" $HOME/$file  || exit 1
-          cat <<EOF
-
-NOTE: $HOME/$file file successfully updated.
-      Backup written to $dotFileBackup
+          cat 1>&2 <<EOF
+NOTE: Successfully updated the following file:
+      $HOME/$file
+      Backup written to:
+      $dotFileBackup
 
 EOF
           ;;
         esac
       else
-        echo "NOTE: No data for this distribution found in $HOME/$file"
-        echo
+        emit_message "NOTE: No data for this distribution found in:"
+        emit_message "      $HOME/$file"
+        emit_message
       fi
     fi
   done
   removeOldFiles
-  echo "Done. Deleted uninstall.sh file."
-  rm -f ./test.sh      || exit 1
-  rm -f ./uninstall.sh || exit 1
+  emit_message "Done. Deleted uninstall.sh file."
+  rm -f $mydir/test.sh      || exit 1
+  rm -f $mydir/uninstall.sh || exit 1
 }
 
 writeUninstallFile() {
-  uninstallFile=./uninstall.sh
-  echo "#!/bin/sh"                                > $uninstallFile || exit 1
-  echo "$PWD/install.sh --uninstall \\"          >> $uninstallFile || exit 1
-  echo "  --catalogManager=$myCatalogManager \\" >> $uninstallFile || exit 1
-  echo "  --dotEmacs=$myEmacsFile"               >> $uninstallFile || exit 1
+  uninstallFile=$mydir/uninstall.sh
+  echo "#!/bin/bash"                               > $uninstallFile || exit 1
+  echo "mydir=\$(readlink -f \$(dirname \$0))"    >> $uninstallFile || exit 1
+  echo "\$mydir/install.sh \\"                    >> $uninstallFile || exit 1
+  echo "  --uninstall \\"                         >> $uninstallFile || exit 1
+  echo "  --catalogManager=$myCatalogManager \\"  >> $uninstallFile || exit 1
+  echo "  --dotEmacs=$myEmacsFile \\"             >> $uninstallFile || exit 1
+  echo "  \$@"                                    >> $uninstallFile || exit 1
   chmod 755 $uninstallFile || exit 1
 }
 
 writeTestFile() {
-  testFile=./test.sh
-  echo "#!/bin/sh"                                > $testFile || exit 1
-  echo "$PWD/install.sh --test"                  >> $testFile || exit 1
+  testFile=$mydir/test.sh
+  echo "#!/bin/bash"                                > $testFile || exit 1
+  echo "mydir=\$(readlink -f \$(dirname \$0))"     >> $testFile || exit 1
+  echo "\$mydir/install.sh --test"                 >> $testFile || exit 1
   chmod 755 $testFile || exit 1
 }
 
 printExitMessage() {
-  cat <<EOF
-
-Type the following to source your shell environment for the distribution
+  cat 1>&2 <<EOF
+To source your shell environment for this distribution, type the
+following:
 
 $appendLine
 
@@ -711,12 +822,13 @@ EOF
 checkForResolver() {
   resolverResponse="$(java org.apache.xml.resolver.apps.resolver uri -u foo 2>/dev/null)"
   if [ -z "$resolverResponse" ]; then
-    cat <<EOF
+    cat 1>&2 <<EOF
 
-NOTE: Your environment does not seem to contain the Apache XML Commons
-      Resolver; without that, you can't use XML catalogs with Java.
-      For more information, see the "How to use a catalog file" section
-      in Bob Stayton's "DocBook XSL: The Complete Guide"
+NOTE: Your environment does not seem to contain the Apache XML
+      Commons Resolver; without that, you can't use XML catalogs
+      with Java applications. For more information, see the "How
+      to use a catalog file" section in Bob Stayton's "DocBook
+      XSL: The Complete Guide"
 
       http://sagehill.net/docbookxsl/UseCatalog.html
 
@@ -725,9 +837,9 @@ EOF
 }
 
 emitNoChangeMsg() {
-  cat <<EOF
+  cat 1>&2 <<EOF
 
-NOTE: No changes was made to CatalogManagers.properties. To
+NOTE: No changes were made to CatalogManagers.properties. To
       provide your Java tools with XML catalog information for
       this distribution, you will need to make the appropriate
       changes manually.
@@ -736,22 +848,30 @@ EOF
 }
 
 testCatalogs() {
-  readlinkResponse="$(readlink -f ./ 2>/dev/null)"
+  readlinkResponse="$(readlink -f . 2>/dev/null)"
   if [ -z "$readlinkResponse" ]; then
-    cat <<EOF
+    cat 1>&2 <<EOF
 
 FATAL: Cannot locate the "readlink" command. Stopping.
 EOF
   exit
   fi
 
+  if [ ! -f "$thisXmlCatalog" ]; then
+    cat 1>&2 <<EOF
+
+FATAL: $thisXmlCatalog file needed but not found. Stopping.
+EOF
+  exit
+  fi
+
   if [ -z "$XML_CATALOG_FILES" ]; then
-    echo
-    echo "WARNING: XML_CATALOG_FILES not set. Not testing with xmlcatalog."
+    emit_message
+    emit_message "WARNING: XML_CATALOG_FILES not set. Not testing with xmlcatalog."
   else
     xmlCatalogResponse="$(xmlcatalog 2>/dev/null)"
     if [ -z "$xmlCatalogResponse" ]; then
-    cat <<EOF
+    cat 1>&2 <<EOF
 
 WARNING: Cannot locate the "xmlcatalog" command. Make sure that
          you have libxml2 and its associated utilities installed.
@@ -760,42 +880,41 @@ WARNING: Cannot locate the "xmlcatalog" command. Make sure that
 
 EOF
     else
-      echo
-      echo "Testing with xmlcatalog..."
+      emit_message "Testing with xmlcatalog..."
       while read pair; do
-        path=$(readlink -f "${pair%* *}")
+        path=$(readlink -f "$mydir/${pair%* *}")
         uri=${pair#* *}
-        echo
-        echo "  Tested:" $uri
+        emit_message
+        emit_message "  Tested: $uri"
         for catalog in $XML_CATALOG_FILES; do
           response="$(readlink -f "$(xmlcatalog $catalog $uri| grep -v "No entry")")"
           if [ -n "$response" ]; then
             if [ "$response" = "$path" ]; then
-              echo "  Result: $path"
+              emit_message "  Result: $path"
               break
             else
-              echo "  Result: FAILED"
+              emit_message "  Result: FAILED"
             fi
           fi
         done
-      done < .urilist
+      done < $mydir/.urilist
     fi
   fi
 
   if [ -z "$CLASSPATH" ]; then
-    echo
-    echo "NOTE: CLASSPATH not set. Not testing with Apache XML Commons Resolver."
+    emit_message
+    emit_message "NOTE: CLASSPATH not set. Not testing with Apache XML Commons Resolver."
   else
     if [ "$(checkForResolver)" ]; then
       checkForResolver
     else
-      echo
-      echo "Testing with Apache XML Commons Resolver..."
+      emit_message
+      emit_message "Testing with Apache XML Commons Resolver..."
       while read pair; do
-        path=$(readlink -f ${pair%* *})
+        path=$(readlink -f "$mydir/${pair%* *}")
         uri=${pair#* *}
-        echo
-        echo "  Tested:" $uri
+        emit_message
+        emit_message "  Tested: $uri"
         if [ ${uri%.dtd} != $uri ]; then
           response="$(java org.apache.xml.resolver.apps.resolver system -s $uri | grep "Result")"
         else
@@ -803,12 +922,13 @@ EOF
         fi
         if [ "$response" ]; then
           if [ "${response#*$path}" != "$response" ]; then
-            echo "  Result: $path"
+            emit_message "  Result: $path"
           else
-            echo "  Result: FAILED"
+            emit_message "  Result: FAILED"
           fi
+          echo
         fi
-      done < .urilist
+      done < $mydir/.urilist
     fi
   fi
 }
@@ -828,7 +948,7 @@ esac
 
 # Copyright
 # ---------
-# Copyright 2005 Michael Smith <smith@sideshowbarker.net>
+# Copyright 2005-2007 Michael(tm) Smith <smith@sideshowbarker.net>
 # 
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
